package cz.cvut.kbss.reporting.rest;

import cz.cvut.kbss.reporting.dto.PersonDto;
import cz.cvut.kbss.reporting.model.Person;
import cz.cvut.kbss.reporting.rest.mapper.DtoMapper;
import cz.cvut.kbss.reporting.rest.util.RestUtils;
import cz.cvut.kbss.reporting.service.repository.PersonService;
import cz.cvut.kbss.reporting.service.security.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

@PreAuthorize("hasRole('ROLE_USER')")
@RestController
@RequestMapping("/persons")
public class PersonController extends AbstractController {

    private final PersonService personService;

    private final SecurityUtils securityUtils;

    private final DtoMapper dtoMapper;

    @Autowired
    public PersonController(PersonService personService, SecurityUtils securityUtils, DtoMapper dtoMapper) {
        this.personService = personService;
        this.securityUtils = securityUtils;
        this.dtoMapper = dtoMapper;
    }

    @PreAuthorize("permitAll()")
    @RequestMapping(method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Void> createPerson(@RequestBody Person person) {
        personService.persist(person);
        if (LOG.isDebugEnabled()) {
            LOG.debug("User {} {} created.", person.getFirstName(), person.getLastName());
        }
        final HttpHeaders headers = RestUtils.createLocationHeaderFromCurrentUri("/{id}", person.getId());
        return new ResponseEntity<>(headers, HttpStatus.CREATED);
    }

    @RequestMapping(value = "/current", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public Person getCurrentUser() {
        return securityUtils.getCurrentUser();
    }

    @RequestMapping(value = "/current", method = RequestMethod.PUT, consumes = MediaType.APPLICATION_JSON_VALUE)
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public void updateCurrentUser(@RequestBody PersonDto dto) {
        if (dto.getPassword() != null) {
            securityUtils.verifyCurrentUserPassword(dto.getPasswordOriginal());
        }
        final Person person = dtoMapper.personDtoToPerson(dto);
        personService.update(person);
        if (LOG.isDebugEnabled()) {
            LOG.debug("Person {} successfully updated.", person);
        }
    }

    @PreAuthorize("permitAll()")
    @RequestMapping(value = "/exists", method = RequestMethod.GET)
    @ResponseBody
    public String desUsernameExist(@RequestParam(name = "username") String username) {
        return Boolean.toString(personService.exists(username));
    }
}
